@@ -53,7 +53,7 @@ Revision History:
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
-#include <iostream.h>
+#include <iostream>
 
 #include "typeapi.h"
 #include "codehead.h"
