@@ -40,7 +40,7 @@ Revision History:
 *************************************************************************/
 
 #include <stdio.h>
-#include <iostream.h>
+#include <iostream>
 #include <math.h>
 #include <stdlib.h>
 
