@@ -37,11 +37,11 @@
  * SUCH DAMAGE.
  */
 
-#ifndef WIN32
+#ifndef _WIN32
 #ifndef _CONFIG_UNIX_H
 #define _CONFIG_UNIX_H
-#include "uclconf.h"
-#include "../../include/mpeg4ip_version.h"
+#include "mpeg4ip.h"
+//#include "../../include/mpeg4ip_version.h"
 #include <limits.h>
 #include <sys/types.h>
 #if TIME_WITH_SYS_TIME
