@@ -38,11 +38,11 @@
 
 #ifndef __RTP_H__
 #define __RTP_H__
-#if 0
-#include "config_unix.h"
-#include "config_win32.h"
-#include "net_udp.h"
+
+#ifndef HAVE_STRUCT_SOCKADDR_STORAGE
+#include "sockstorage.h"
 #endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -53,8 +53,8 @@ extern "C" {
 
 
   // moved here by nori
-typedef int (*rtp_encrypt_f)(void *, uint8_t *, unsigned int *);
-typedef int (*rtp_decrypt_f)(void *ud, uint8_t *packet_head, unsigned int *packet_len);
+typedef int (*rtp_encrypt_f)(void *, uint8_t *, uint32_t *);
+typedef int (*rtp_decrypt_f)(void *ud, uint8_t *packet_head, uint32_t *packet_len);
 
 struct rtp;
 
@@ -62,20 +62,6 @@ struct rtp;
  * struct *'s. */
 typedef struct rtp *rtp_t;
 
-typedef struct rtp_packet_data {
-  struct rtp_packet *rtp_pd_next, *rtp_pd_prev;
-  uint32_t	*rtp_pd_csrc;
-  uint8_t	*rtp_pd_data;
-  uint32_t	 rtp_pd_data_len;
-  uint8_t	*rtp_pd_extn;
-  uint16_t	 rtp_pd_extn_len; /* Size of the extension in 32 bit words minus one */
-  uint16_t	 rtp_pd_extn_type;/* Extension type field in the RTP packet header   */
-  uint32_t       rtp_pd_buflen; /* received buffer len (w/rtp header) */
-  int            rtp_pd_have_timestamp;
-  uint64_t       rtp_pd_timestamp;
-} rtp_packet_data;
-
-  
 typedef struct rtp_packet_header {
 #ifdef WORDS_BIGENDIAN
 	unsigned short   ph_v:2;	/* packet type                */
@@ -99,6 +85,27 @@ typedef struct rtp_packet_header {
 	/* be represented in the struct.                              */
 } rtp_packet_header;
 
+typedef struct rtp_packet_data {
+  struct rtp_packet *rtp_pd_next, *rtp_pd_prev;
+  uint32_t	*rtp_pd_csrc;
+  uint8_t	*rtp_pd_data;
+  uint32_t	 rtp_pd_data_len;
+  uint8_t	*rtp_pd_extn;
+  uint16_t	 rtp_pd_extn_len; /* Size of the extension in 32 bit words minus one */
+  uint16_t	 rtp_pd_extn_type;/* Extension type field in the RTP packet header   */
+  uint8_t       *pd_buf_start;
+  uint32_t       pd_buflen; /* received buffer len (w/rtp header) */
+  int            rtp_pd_have_timestamp;
+  uint64_t       rtp_pd_timestamp;
+  struct sockaddr_storage rtp_rx_addr;
+  socklen_t      rtp_rx_addr_len;
+  rtp_packet_header *ph;
+} rtp_packet_data;
+
+  
+
+#define packet_start    pd.pd_buf_start
+#define packet_length   pd.pd_buflen
 #define rtp_next      pd.rtp_pd_next
 #define rtp_prev      pd.rtp_pd_prev
 #define rtp_csrc      pd.rtp_pd_csrc
@@ -108,15 +115,15 @@ typedef struct rtp_packet_header {
 #define rtp_extn_len  pd.rtp_pd_extn_len
 #define rtp_extn_type pd.rtp_pd_extn_type
   
-#define rtp_pak_v    ph.ph_v
-#define rtp_pak_p    ph.ph_p
-#define rtp_pak_x    ph.ph_x
-#define rtp_pak_cc   ph.ph_cc
-#define rtp_pak_m    ph.ph_m
-#define rtp_pak_pt   ph.ph_pt
-#define rtp_pak_seq  ph.ph_seq
-#define rtp_pak_ts   ph.ph_ts
-#define rtp_pak_ssrc ph.ph_ssrc
+#define rtp_pak_v    pd.ph->ph_v
+#define rtp_pak_p    pd.ph->ph_p
+#define rtp_pak_x    pd.ph->ph_x
+#define rtp_pak_cc   pd.ph->ph_cc
+#define rtp_pak_m    pd.ph->ph_m
+#define rtp_pak_pt   pd.ph->ph_pt
+#define rtp_pak_seq  pd.ph->ph_seq
+#define rtp_pak_ts   pd.ph->ph_ts
+#define rtp_pak_ssrc pd.ph->ph_ssrc
 
 typedef struct rtp_packet {
   /* The following are pointers to the data in the packet as    */
@@ -127,7 +134,6 @@ typedef struct rtp_packet {
   /* having to free the csrc, data and extn blocks separately.  */
   rtp_packet_data pd;
   /* The following map directly onto the RTP packet header...   */
-  rtp_packet_header ph;
 } rtp_packet;
   
 typedef struct {
@@ -197,8 +203,9 @@ typedef struct {
 /* Callback types */
 typedef void (*rtp_callback_f)(struct rtp *session, rtp_event *e);
 typedef rtcp_app* (*rtcp_app_callback_f)(struct rtp *session, uint32_t rtp_ts, uint32_t max_size);
-  typedef int (*rtcp_send_packet_f)(void *userdata, uint8_t *buffer, uint32_t buflen);
 
+typedef int (*send_packet_f)(void *userdata, uint8_t *buffer, uint32_t buflen);
+typedef int (*send_packet_iov_f)(void *userdata, struct iovec *iov, int count);
 /* SDES packet types... */
 typedef enum  {
         RTCP_SDES_END   = 0,
@@ -225,6 +232,8 @@ typedef enum {
         RTP_OPT_FILTER_MY_PACKETS = 3
 } rtp_option;
 
+typedef struct socket_udp_ socket_udp; 
+
 typedef struct rtp_stream_params_t {
   const char *rtp_addr;
   in_port_t   rtp_rx_port;
@@ -242,13 +251,16 @@ typedef struct rtp_stream_params_t {
 
   double rtcp_bandwidth;  
   
-  int  rtp_socket;  // rtp socket to use
-  int  rtcp_socket; // rtcp socket to use
+  socket_udp  *rtp_socket;  // rtp socket to use
+  socket_udp  *rtcp_socket; // rtcp socket to use
   // callbacks
   rtp_callback_f rtp_callback;
-  rtcp_send_packet_f rtcp_send_packet;
+  send_packet_f rtp_send_packet;
+  send_packet_iov_f rtp_send_packet_iov;
+  send_packet_f rtcp_send_packet;
   // user data
-  void *userdata;
+  void *send_userdata;
+  void *recv_userdata;
 } rtp_stream_params_t;
 
 typedef struct rtp_encryption_params_t {
@@ -269,25 +281,14 @@ rtp_t		rtp_init(const char *addr,
 			 uint16_t rx_port, uint16_t tx_port, 
 			 int ttl, double rtcp_bw, 
 			 rtp_callback_f callback,
-			 uint8_t *userdata);
+			 void *recv_userdata);
   // rtp_init_xmitter - for transmitters - send an RTCP with the first packet
-rtp_t		rtp_init_xmitter(const char *addr, 
-				 uint16_t rx_port, uint16_t tx_port, 
-				 int ttl, double rtcp_bw, 
-				 rtp_callback_f callback,
-				 uint8_t *userdata);
 rtp_t		rtp_init_if(const char *addr, char *iface, 
 			    uint16_t rx_port, uint16_t tx_port, 
 			    int ttl, double rtcp_bw, 
 			    rtp_callback_f callback,
-			    uint8_t *userdata,
+			    void *recv_userdata,
 			    int dont_init_sockets);
-rtp_t		rtp_init_extern_net(const char *addr, 
-				    uint16_t rx_port, uint16_t tx_port, 
-				    int ttl, double rtcp_bw, 
-				    rtp_callback_f callback,
-				    rtcp_send_packet_f rtcp_send_packet,
-				    uint8_t *userdata);
 
 void		 rtp_send_bye(struct rtp *session);
 void		 rtp_done(struct rtp *session);
@@ -303,14 +304,12 @@ int 		 rtp_send_data(struct rtp *session,
 			       unsigned int cc, uint32_t csrc[], 
                                uint8_t *data, uint32_t data_len, 
 			       uint8_t *extn, uint16_t extn_len, uint16_t extn_type);
-#ifndef _WIN32
 int            rtp_send_data_iov(struct rtp *session, 
 				 uint32_t rtp_ts, int8_t pt, int m, 
 				 unsigned int cc, uint32_t csrc[], 
 				 struct iovec *iov, uint32_t iov_count, 
 				 uint8_t *extn, uint16_t extn_len, 
 				 uint16_t extn_type, uint16_t seq_num_add);
-#endif
 void 		 rtp_send_ctrl(struct rtp *session, uint32_t rtp_ts, 
 			       rtcp_app_callback_f appcallback);
 void 		 rtp_send_ctrl_2(struct rtp *session, uint32_t rtp_ts,
@@ -330,39 +329,34 @@ const char	*rtp_get_sdes(struct rtp *session, uint32_t ssrc, rtcp_sdes_type type
 const rtcp_sr	*rtp_get_sr(struct rtp *session, uint32_t ssrc);
 const rtcp_rr	*rtp_get_rr(struct rtp *session, uint32_t reporter, uint32_t reportee);
 
-int              rtp_set_encryption_key(struct rtp *session, const char *passphrase);
 int              rtp_set_my_ssrc(struct rtp *session, uint32_t ssrc);
 
 char 		*rtp_get_addr(struct rtp *session);
 uint16_t	 rtp_get_rx_port(struct rtp *session);
 uint16_t	 rtp_get_tx_port(struct rtp *session);
 int		 rtp_get_ttl(struct rtp *session);
-uint8_t		*rtp_get_userdata(struct rtp *session);
+void		*rtp_get_recv_userdata(struct rtp *session);
 
   
 int rtp_process_recv_data(struct rtp *session,
 			  uint32_t curr_rtp_ts,
-			  rtp_packet *packet,
-			  uint32_t buflen);
+			  rtp_packet *packet);
 
 void rtp_process_ctrl(struct rtp *session, uint8_t *buffer, uint32_t buflen);
 
-  // added by nori
-  int rtp_set_encryption(struct rtp *session, 
-		       rtp_encrypt_f rtp_efunc, rtp_decrypt_f rtp_dfunc , 
-		       rtp_encrypt_f rtcp_efunc, rtp_decrypt_f rtcp_dfunc,
-		       void *userdata, unsigned int rtp_payload_add);
-  
   int rtp_set_encryption_params (struct rtp *session, rtp_encryption_params_t *params);
   
   int rtp_get_encryption_enabled(struct rtp *session);
   
-typedef struct socket_udp_ socket_udp; 
 
 socket_udp *get_rtp_data_socket(struct rtp *session);
 socket_udp *get_rtp_rtcp_socket(struct rtp *session);
 
   void rtp_set_receive_buffer_default_size(int bufsize);  
+  unsigned int rtp_get_mtu_adjustment (struct rtp *session);
+
+  void rtp_set_rtp_callback(struct rtp *session, rtp_callback_f rtp,
+			    void *userdata);
 #ifdef __cplusplus
 }
 #endif
