@@ -107,17 +107,15 @@ typedef struct key_desc_t {
   const char *key;
 } key_desc_t;
 
-typedef struct rtpmap_desc_t {
-  const char *encode_name;
-  uint32_t clock_rate;
-  uint32_t encode_param;
-} rtpmap_desc_t;
-
 typedef struct format_list_t {
   struct format_list_t *next;
   struct media_desc_t *media;
   const char *fmt;
-  rtpmap_desc_t *rtpmap;
+  /* rtp map parameters */
+  const char *rtpmap_name;
+  uint32_t rtpmap_clock_rate;
+  uint32_t rtpmap_encode_param;
+  /* fmtp line */
   const char *fmt_param;
 } format_list_t;
 
@@ -156,7 +154,7 @@ typedef struct media_desc_t {
   const char *lang;
   const char *media_desc;      // description string
   const char *control_string;  // rtsp control string
-  format_list_t *fmt;    // All possible formats for this media
+  format_list_t *fmt_list;    // All possible formats for this media
   string_list_t *unparsed_a_lines;  // Any unparsed lines
   int recvonly, sendrecv, sendonly;
   uint16_t port;       // ip port
@@ -222,6 +220,7 @@ typedef struct session_desc_t {
   /* Administrator Information */
   string_list_t *admin_phone;
   string_list_t *admin_email;
+  bool no_admin_or_phone;
   /* connect info */
   connect_desc_t session_connect;
   range_desc_t   session_range;
@@ -273,6 +272,7 @@ int sdp_decode(sdp_decode_info_t *decode,
 	       int *translated);
 
 void sdp_free_session_desc(session_desc_t *sptr);
+  void sdp_free_media_desc(media_desc_t *mptr);
 
 /*
  * dump routines
@@ -301,14 +301,18 @@ void sdp_set_error_func(error_msg_func_t func);
   
 /* utils */
 format_list_t *sdp_add_format_to_list(media_desc_t *mptr, const char *val);
+  format_list_t *sdp_add_format_to_end_of_list(format_list_t *list,
+					       format_list_t *newone);
 void sdp_free_format_list (format_list_t **fptr);
 
 int sdp_add_string_to_list(string_list_t **list, const char *val);
+  int sdp_add_strings_to_list(string_list_t **list, const char *val);
 void sdp_free_string_list (string_list_t **list);
 
 void sdp_time_offset_to_str(uint32_t val, char *buff, uint32_t buflen);
 format_list_t *sdp_find_format_in_line(format_list_t *head, const char *lptr);
-void sdp_smpte_to_str(double value, uint16_t fps, char *buffer);
+format_list_t *sdp_find_rtpmap_name(format_list_t *head, const char *name);
+void sdp_smpte_to_str(double value, uint16_t fps, char *buffer, uint buflen);
 
 media_desc_t *sdp_find_media_type(session_desc_t *sptr, const char *name);
 const char *find_unparsed_a_value(string_list_t *lptr, const char *value);
