@@ -44,6 +44,22 @@
  *   pointer to added value.  This could be a new one, or one previously
  *   added
  */
+format_list_t *sdp_add_format_to_end_of_list(format_list_t *list,
+					     format_list_t *newone)
+{
+  format_list_t *p;
+  if (list == NULL) {
+    list = newone;
+  } else {
+    p = list;
+    while (p->next != NULL) {
+      p = p->next;
+    }
+    p->next = newone;
+  }
+  return list;
+}
+
 format_list_t *sdp_add_format_to_list (media_desc_t *mptr, const char *val)
 {
   format_list_t *new, *p;
@@ -55,7 +71,9 @@ format_list_t *sdp_add_format_to_list (media_desc_t *mptr, const char *val)
 
   new->next = NULL;
   new->fmt = strdup(val);
-  new->rtpmap = NULL;
+  new->rtpmap_name = NULL;
+  new->rtpmap_clock_rate = 0;
+  new->rtpmap_encode_param = 0;
   new->fmt_param = NULL;
   new->media = mptr;
   
@@ -64,17 +82,19 @@ format_list_t *sdp_add_format_to_list (media_desc_t *mptr, const char *val)
     return (NULL);
   }
   
-  if (mptr->fmt == NULL) {
-    mptr->fmt = new;
+  if (mptr->fmt_list == NULL) {
+    mptr->fmt_list = new;
   } else {
-    p = mptr->fmt;
+    p = mptr->fmt_list;
     if (strcmp(p->fmt, new->fmt) == 0) {
+      CHECK_AND_FREE(new->fmt);
       free(new);
       return (p);
     }
     while (p->next != NULL) {
       p = p->next;
       if (strcmp(p->fmt, new->fmt) == 0) {
+	CHECK_AND_FREE(new->fmt);
 	free(new);
 	return (p);
       }
@@ -84,12 +104,6 @@ format_list_t *sdp_add_format_to_list (media_desc_t *mptr, const char *val)
   return (new);
 }
 
-static void free_rtpmap_desc (rtpmap_desc_t *rtpptr)
-{
-  FREE_CHECK(rtpptr, encode_name);
-  free(rtpptr);
-}
-				   
 void sdp_free_format_list (format_list_t **fptr)
 {
   format_list_t *p;
@@ -98,10 +112,7 @@ void sdp_free_format_list (format_list_t **fptr)
     p = *fptr;
     *fptr = p->next;
     p->next = NULL;
-    if (p->rtpmap != NULL) {
-      free_rtpmap_desc(p->rtpmap);
-      p->rtpmap = NULL;
-    }
+    FREE_CHECK(p, rtpmap_name);
     FREE_CHECK(p, fmt_param);
     FREE_CHECK(p, fmt);
     free(p);
@@ -143,6 +154,48 @@ int sdp_add_string_to_list (string_list_t **list, const char *val)
   return (TRUE);
 }
 
+int sdp_add_strings_to_list (string_list_t **list, const char *val)
+{
+  string_list_t *new, *p;
+  const char *end;
+  char *string_val;
+  bool added = FALSE;
+  if (val == NULL) return TRUE;
+  while (*val != '\0') {
+    end = val;
+    while (*end != '\0' && *end != '\n' && *end != '\n') end++;
+    if (end == val) {
+      return added;
+    }
+    added = TRUE;
+    new = malloc(sizeof(string_list_t));
+    if (new == NULL) {
+      return (FALSE);
+    }
+    
+    new->next = NULL;
+    string_val = malloc(1 + end - val);
+    if (string_val == NULL) {
+      free(new);
+      return (FALSE);
+    }
+    memcpy(string_val, val, end - val);
+    string_val[end - val] = '\0';
+    new->string_val = string_val;
+    
+    if (*list == NULL) {
+      *list = new;
+    } else {
+      p = *list;
+      while (p->next != NULL) p = p->next;
+      p->next = new;
+    }
+    val = end;
+    while (*val == '\n' || *val == '\r' || isspace(*val)) val++;
+  }
+  return (TRUE);
+}
+
 void sdp_free_string_list (string_list_t **list)
 {
   string_list_t *p;
@@ -214,7 +267,21 @@ format_list_t *sdp_find_format_in_line (format_list_t *head, const char *lptr)
   return (NULL);
 }
 
-void sdp_smpte_to_str (double value, uint16_t fps, char *buffer)
+format_list_t *sdp_find_rtpmap_name (format_list_t *head, 
+				     const char *name)
+{
+  while (head != NULL) {
+    if (head->rtpmap_name != NULL &&
+	(strcasecmp(name, head->rtpmap_name) == 0)) {
+      return (head);
+    } else 
+      head = head->next;
+  }
+  return (NULL);
+}
+
+void sdp_smpte_to_str (double value, uint16_t fps, char *buffer, 
+		       uint buflen)
 {
   double divval;
   unsigned int temp;
@@ -227,14 +294,16 @@ void sdp_smpte_to_str (double value, uint16_t fps, char *buffer)
     temp++;
     value -= divval;
   }
-  ix = sprintf(buffer, "%02d:", temp);
+  ix = snprintf(buffer, buflen, "%02d:", temp);
+  if (ix > buflen) return;
   temp = 0;
   divval = 60.0 * fps;
   while (value >= divval) {
     temp++;
     value -= divval;
   }
-  ix += sprintf(buffer + ix, "%02d:", temp);
+  ix += snprintf(buffer + ix, buflen - ix, "%02d:", temp);
+  if (ix > buflen) return;
 
   temp = 0;
   divval = fps;
@@ -242,8 +311,9 @@ void sdp_smpte_to_str (double value, uint16_t fps, char *buffer)
     temp++;
     value -= divval;
   }
-  ix += sprintf(buffer + ix, "%02d", temp);
-  if (value > 0.0) sprintf(buffer + ix, ":%02g", value);
+  ix += snprintf(buffer + ix, buflen - ix, "%02d", temp);
+  if (ix > buflen) return;
+  if (value > 0.0) snprintf(buffer + ix, buflen - ix, ":%02g", value);
 }
 
 static int sdp_debug_level = LOG_ALERT;
