@@ -131,7 +131,7 @@ static int parse_vovod (iso_decode_t *iso,
     uint8_t *write;
     write = buffer;
     // Convert the config= from ascii to binary
-    for (uint32_t ix = 0; ix < len; ix++) {
+    for (uint32_t ix = 0; ix < len; ix += 2) {
       *write = 0;
       *write = (tohex(*config)) << 4;
       config++;
@@ -605,9 +605,9 @@ static int iso_codec_check (lib_message_func_t message,
       ret_val =  1;
     }
 #endif
-    if (fptr->rtpmap != NULL && fptr->rtpmap->encode_name != NULL) {
-      if (strcasecmp(fptr->rtpmap->encode_name, "MP4V-ES") == 0 || 
-          strcasecmp(fptr->rtpmap->encode_name, "enc-mpeg4-generic") == 0) {
+    if (fptr->rtpmap_name != NULL) {
+      if (strcasecmp(fptr->rtpmap_name, "MP4V-ES") == 0 || 
+          strcasecmp(fptr->rtpmap_name, "enc-mpeg4-generic") == 0) {
 	ret_val =  1;
       }
     }
