@@ -125,8 +125,8 @@ static int create_media_from_sdp (CPlayerSession *psptr,
   if (sdp->media != NULL &&
       sdp->media->next == NULL &&
       strcasecmp(sdp->media->media, "video") == 0 &&
-      sdp->media->fmt != NULL &&
-      strcmp(sdp->media->fmt->fmt, "33") == 0) {
+      sdp->media->fmt_list != NULL &&
+      strcmp(sdp->media->fmt_list->fmt, "33") == 0) {
     // we have a mpeg2 transport stream
     return (create_mpeg2t_session(psptr, NULL, sdp, 
 				  have_audio_driver, cc_vft));
@@ -179,7 +179,7 @@ static int create_media_from_sdp (CPlayerSession *psptr,
 
     if (have_audio_driver != 0 &&
 	strcasecmp(sdp_media->media, "audio") == 0) {
-      fmt = sdp_media->fmt;
+      fmt = sdp_media->fmt_list;
       codec = NULL;
       while (codec == NULL && fmt != NULL) {
 	codec = check_for_audio_codec(STREAM_TYPE_RTP,
@@ -215,7 +215,7 @@ static int create_media_from_sdp (CPlayerSession *psptr,
 	audio_offset++;
       }
     } else if (strcasecmp(sdp_media->media, "video") == 0) {
-	fmt = sdp_media->fmt;
+	fmt = sdp_media->fmt_list;
 	codec = NULL;
 	while (codec == NULL && fmt != NULL) {
 	  codec = check_for_video_codec(STREAM_TYPE_RTP,
@@ -252,7 +252,7 @@ static int create_media_from_sdp (CPlayerSession *psptr,
 	}
       } else if (strcasecmp(sdp_media->media, "application") == 0 ||
 		 strcasecmp(sdp_media->media, "control") == 0) {
-	fmt = sdp_media->fmt;
+	fmt = sdp_media->fmt_list;
 	codec = NULL;
 	while (codec == NULL && fmt != NULL) {
 	  codec = check_for_text_codec(STREAM_TYPE_RTP, 
@@ -268,7 +268,7 @@ static int create_media_from_sdp (CPlayerSession *psptr,
 	      fmt = fmt->next;
 	    }
 	  } else {
-	    if (fmt != sdp_media->fmt || fmt->next != NULL) {
+	    if (fmt != sdp_media->fmt_list || fmt->next != NULL) {
 	      player_error_message("Disallow sdp stream %s - more than 1 possible format", sdp_media->media);
 	      codec = NULL;
 	      fmt = fmt->next;
@@ -318,7 +318,8 @@ static int create_media_from_sdp (CPlayerSession *psptr,
       err = mptr->create_streaming(vq[ix].fptr->media, 
 				   broadcast, 
 				   config.get_config_value(CONFIG_USE_RTP_OVER_RTSP),
-				   media_count);
+				   media_count, 
+				   ix == 0 ? psptr->get_video_rtp_session() : NULL);
       if (err < 0) {
 	return (-1);
       }
@@ -334,7 +335,8 @@ static int create_media_from_sdp (CPlayerSession *psptr,
       err = mptr->create_streaming(aq[ix].fptr->media, 
 				   broadcast, 
 				   config.get_config_value(CONFIG_USE_RTP_OVER_RTSP),
-				   media_count);
+				   media_count,
+				   ix == 0 ? psptr->get_audio_rtp_session() : NULL);
       if (err < 0) {
 	return (-1);
       }
@@ -350,7 +352,8 @@ static int create_media_from_sdp (CPlayerSession *psptr,
       err = mptr->create_streaming(tq[ix].fptr->media, 
 				   broadcast, 
 				   config.get_config_value(CONFIG_USE_RTP_OVER_RTSP),
-				   media_count);
+				   media_count,
+				   NULL);
       if (err < 0) {
 	return (-1);
       }
@@ -694,7 +697,7 @@ int parse_name_for_session (CPlayerSession *psptr,
   }
 #endif
 
-  if (strncmp(name, "v=0\r\n", strlen("v=0\r\n")) == 0) {
+  if (strncmp(name, "v=0", strlen("v=0")) == 0) {
     // sdp
     err = create_media_from_sdp(psptr, 
 				name, 
@@ -775,6 +778,7 @@ int parse_name_for_session (CPlayerSession *psptr,
 				    cc_vft);
   } else if (strcasecmp(suffix, ".mpeg") == 0 ||
 	     strcasecmp(suffix, ".mpg") == 0 ||
+	     strcasecmp(suffix, ".ts") == 0 ||
 	     strcasecmp(suffix, ".vob") == 0) {
 #ifdef _WIN32
 	  err = -1;
@@ -939,7 +943,9 @@ CPlayerSession *start_session (CMsgQueue *master_queue,
 			       int screen_loc_y,
 			       int screen_size,
 			       double start_time,
-			       bool start_thread)
+			       struct rtp *video_rtp,
+			       struct rtp *audio_rtp
+			       )
 {
   CPlayerSession *psptr;
 
@@ -955,6 +961,8 @@ CPlayerSession *start_session (CMsgQueue *master_queue,
 			     start_time);
   if (psptr == NULL) return NULL;
 
+  psptr->set_audio_rtp_session(audio_rtp);
+  psptr->set_video_rtp_session(video_rtp);
   psptr->set_audio_volume(audio_volume);
   psptr->set_screen_location(screen_loc_x, screen_loc_y);
   bool fullscreen = config.GetBoolValue(CONFIG_FULL_SCREEN);
